@@ -43,7 +43,7 @@ from sphinx.util import ensuredir, relative_uri
 OUTPUT_DEFAULT_FORMATS = dict(html='svg', latex='pdf', text=None)
 OWN_OPTION_SPEC = dict( { 'caption': str,
     'size': str,
-    'suffix': str,
+    'plot_format': str,
     'convert': str,
     'show_source': str,
     })
@@ -152,32 +152,28 @@ def cmd_2_image (app, plot):
     #print("app.builder.imagedir: %s" %(app.builder.imagedir))
 
     cmd = plot['cmd']
+    args = shlex.split(cmd)
     text = plot['text']
     options = plot['options']
     format = app.builder.format
-    cmd_args = shlex.split(cmd)
     rel_imgpath = relative_uri(app.builder.env.docname, app.builder.imagedir)
     hashkey = cmd + str(options) + text
     hashkey = sha(hashkey.encode('utf-8')).hexdigest()
-    infname = '%s-%s.%s' % (cmd_args[0], hashkey, plot['suffix'])
+    infname = '%s-%s.%s' % (args[0], hashkey, plot['suffix'])
     infullfn = path.join(app.builder.outdir, app.builder.imagedir, infname)
     ensuredir(path.join(app.builder.outdir, app.builder.imagedir))
     currpath = os.getcwd() # Record the current dir and return here afterwards
 
-    if options.get("suffix", None): # User definition is the 1st choice.
-        suffix = options["suffix"]
-    elif ((cmd_args[0] == "gnuplot") or
-            ((cmd_args[0] == "ditaa") and
-                ('--svg' in cmd_args) and
-                not options.get("convert", None))):
-        # Now: gnuplot and ditaa support vector image: .svg, .pdf
+    if options.get("plot_format", None): #User definition is higher priority.
+        plot_format = options["plot_format"]
+    else:
         format_map = OUTPUT_DEFAULT_FORMATS.copy()
         format_map.update(app.builder.config.plot_output_format)
-        suffix = format_map.get(app.builder.format, "png")
-    else:
-        suffix = "png"
+        plot_format = format_map.get(app.builder.format, "png")
+    # convert only support png.
+    plot_format = options.get("convert", None) and "png" or plot_format
 
-    outfname = '%s-%s.%s' %(cmd_args[0], hashkey, suffix)
+    outfname = '%s-%s.%s' %(args[0], hashkey, plot_format)
     out = dict(outrelfn = posixpath.join(rel_imgpath, outfname),
         outfullfn = path.join(app.builder.outdir, app.builder.imagedir, outfname),
         #outreference = posixpath.join(rel_imgpath, infname),
@@ -187,69 +183,64 @@ def cmd_2_image (app, plot):
     if path.isfile(out["outfullfn"]):
         print("file has already existed: %s" %(outfname))
         return out
-
-    if cmd_args[0] == "ditaa":
-        cmd_args.extend([infname, outfname])
+    if "ditaa" in cmd:
+        if plot_format in ["svg", "pdf"]:
+            cmd += " --svg" #ditaa support vector image by --svg parameter.
+        args.extend([infname, outfname])
         # Ditaa must work on the target directory.
         os.chdir(path.join(app.builder.outdir, app.builder.imagedir))
-    elif cmd_args[0] == "dot":
+    elif args[0] == "dot":
         # dot -Tpng in_file -o out_file
-        cmd_args.extend([infullfn, '-o', out["outfullfn"]])
-    elif cmd_args[0] == "python" or cmd_args[0] == "python3" or cmd_args[0] == "python2":
-        # change the embeded output file.
-        text = re.sub(r'savefig\((.+)\)',
-                r'savefig("%s")' %(out["outfullfn"]),
-                text)
-        cmd_args.append(infullfn)
-    elif cmd_args[0] == "gnuplot":
-        size = None
-        if ('size' in options):
-            if (suffix in ["pdf", "eps"]):
-                if "," in options["size"]:
-                    # pdf unit is inch while png is pixel, convert them.
-                    tmp = map(lambda x: int(x.strip())/100, options["size"].split(","))
-                    size = ",".join("%d"%(i) for i in tmp)
-                else:
-                    print("size %s is not valid." %(options["size"]))
-            else:
-                size = options["size"]
-        #pdf/pdfcairo/epscairo
-        t = suffix
-        term = size and ("%s size %s" %(t, size)) or t
-        #print("term: %s" %(term))
+        args.extend([infullfn, '-o', out["outfullfn"]])
+    elif "python" in args[0]:
+        pylib = "pyplot"
+        lines = StringIO(text).readlines()
+        for l in lines:
+            if (not l.lstrip().startswith('#')) and ("import matplotlib.pyplot" in l):
+                # Find out pyplot module name, use pyplot if not found.
+                result = re.search("(?<=import matplotlib.pyplot as )\w+", l, flags=0)
+                pylib = result and result.group() or "pyplot"
+            elif ('.show()' in l or 'savefig(' in l):
+                lines.remove(l)
+        lines.append('%s.savefig("%s")\n' %(pylib, out["outfullfn"]))
+        text = ''.join(lines)
+        args.append(infullfn)
+        #print("text: %s, pylib: %s" %(text, pylib))
+    elif args[0] == "gnuplot":
+        size = options.get("size", "900,600")
+        if (plot_format in ["pdf", "eps"] and "," in options["size"]):
+            # pdf unit is inch while png is pixel, convert them.
+            tmp = map(lambda x: int(x.strip())/100, options["size"].split(","))
+            size = ",".join("%d"%(i) for i in tmp)
 
-        # change the embeded output file, suffix is terminal name.
-        #print("^\s*[^#].*output.*$" in text)
-        if ("^\s*[^#].*output.*$" in text):
-            text = re.sub(r'set\s+output (.+)',
-                    r'set output "%s"' %(out["outfullfn"]), text)
-        else:
-            text = 'set output "%s"\n' %(out["outfullfn"]) + text
+        lines = StringIO(text).readlines()
+        for l in lines: # reset the output/terminal lines
+            if ('set\s+output' in l or 'set\s+term' in l):
+                lines.remove(l)
+        lines.insert(0, 'set output "%s"\n' %(out["outfullfn"]))
+        lines.insert(0, 'set terminal %s size %s\n' %(plot_format, size))
+        text = ''.join(lines)
+        #print("text: %s" %(text))
 
-        #print("^\s*[^#].*terminal.*$" in text)
-        if ("^\s*[^#].*terminal.*$" in text):
-            text = re.sub(r'set\s+term(.*)', r'set term %s' %(term), text)
-        else:
-            text = 'set terminal %s\n' %(term) + text
-
-        cmd_args.append(infullfn)
-    elif cmd_args[0] == "convert":
+        args.append(infullfn)
+    elif args[0] == "convert":
         # Attach the body and change the embeded output file.
         if text:
             for i in StringIO(text).readlines():
-                cmd_args.extend(shlex.split(i.rstrip("\r\n\\")))
-        cmd_args[-1] = out["outfullfn"]
+                args.extend(shlex.split(i.rstrip("\r\n\\")))
+        args[-1] = out["outfullfn"]
     else:
-        cmd_args.append(infullfn)
+        args.append(infullfn)
 
+    #print("text: %s" %(text))
     # write the text as infile.
     with open(infullfn, 'wb') as f:
         f.write(text.encode('utf-8'))
 
     # 2) generate the output file
     try:
-        print(' '.join(cmd_args))
-        p = Popen(cmd_args, stdout=PIPE, stdin=PIPE, stderr=PIPE)
+        print(' '.join(args))
+        p = Popen(args, stdout=PIPE, stdin=PIPE, stderr=PIPE)
         stdout, stderr = (p.stdout.read().decode("utf-8"),
                 p.stderr.read().decode("utf-8"))
         print("[31m%s[1;30m%s[0m" %(stderr, stdout))
@@ -266,12 +257,12 @@ def cmd_2_image (app, plot):
         c += " %s" %(out["outfullfn"])
         print(c)
         os.system(c)
-    if (cmd_args[0] == "ditaa") and (suffix in ['pdf']):
+    if (args[0] == "ditaa") and (plot_format in ['pdf']):
         # In fact ditaa don't support pdf, we convert the .svg to .pdf.
-        print("mv %s %s-%s.svg" %(outfname, cmd_args[0], hashkey))
-        os.system("mv %s %s-%s.svg" %(outfname, cmd_args[0], hashkey))
-        print("inkscape -f %s-%s.svg -A %s" %(cmd_args[0], hashkey, out["outfullfn"]))
-        os.system("inkscape -f %s-%s.svg -A %s" %(cmd_args[0], hashkey, out["outfullfn"]))
+        print("mv %s %s-%s.svg" %(outfname, args[0], hashkey))
+        os.system("mv %s %s-%s.svg" %(outfname, args[0], hashkey))
+        print("inkscape -f %s-%s.svg -A %s" %(args[0], hashkey, out["outfullfn"]))
+        os.system("inkscape -f %s-%s.svg -A %s" %(args[0], hashkey, out["outfullfn"]))
 
 
     # 3) Check if it's need to generate the outreference file
